<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Multiple Wishlist GraphQL for Magento 2
 */

namespace Amasty\MultipleWishlistGraphqlByAmasty\Model\Resolver;

use Amasty\MWishlist\Api\Data\WishlistInterface;
use Amasty\MWishlist\Model\Repository\WishlistRepository;
use Magento\Framework\Exception\CouldNotSaveException;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\GraphQl\Config\Element\Field;
use Magento\Framework\GraphQl\Exception\GraphQlAuthorizationException;
use Magento\Framework\GraphQl\Exception\GraphQlInputException;
use Magento\Framework\GraphQl\Exception\GraphQlNoSuchEntityException;
use Magento\Framework\GraphQl\Query\Resolver\ArgumentsProcessorInterface;
use Magento\Framework\GraphQl\Query\Resolver\ContextInterface;
use Magento\Framework\GraphQl\Query\ResolverInterface;
use Magento\Framework\GraphQl\Schema\Type\ResolveInfo;
use Magento\Wishlist\Model\Item;
use Magento\Catalog\Model\ProductRepository;

class AddProductsToWishList implements ResolverInterface
{
    /**
     * @var WishlistRepository
     */
    private $wishlistRepository;

    /**
     * @var ProductRepository
     */
    private $productRepository;

    /**
     * @var ArgumentsProcessorInterface
     */
    private $argsSelection;

    public function __construct(
        WishlistRepository $wishlistRepository,
        ProductRepository $productRepository,
        ArgumentsProcessorInterface $argsSelection
    ) {
        $this->wishlistRepository = $wishlistRepository;
        $this->productRepository = $productRepository;
        $this->argsSelection = $argsSelection;
    }

    /**
     * @param Field $field
     * @param ContextInterface $context
     * @param ResolveInfo $info
     * @param array|null $value
     * @param array|null $args
     * @return WishlistInterface|Item|string
     * @throws GraphQlInputException
     * @throws CouldNotSaveException
     * @throws GraphQlNoSuchEntityException
     * @throws GraphQlAuthorizationException
     */
    public function resolve(
        Field $field,
        $context,
        ResolveInfo $info,
        array $value = null,
        array $args = null
    ) {
        $processedArgs = $this->argsSelection->process($info->fieldName, $args);
        $inputData = $processedArgs['input'];
        $wishlistId = $processedArgs['wishlistId'];
        $customerId = $context->getUserId();

        if (null === $customerId || 0 === $customerId) {
            throw new GraphQlAuthorizationException(__('The current user cannot perform operations on wishlist'));
        }

        try {
            $this->wishlistRepository->addProductToWishlist(
                $wishlistId['wishlist_id'],
                $this->getProducts($inputData),
                $customerId
            );

            return  $this->getWishlistWithItems($wishlistId['wishlist_id']);
        } catch (CouldNotSaveException $exception) {
            throw new GraphQlInputException(
                __("Unable to add products. Error: %1", $exception->getMessage()),
            );
        } catch (NoSuchEntityException $e) {
            throw new GraphQlNoSuchEntityException(__("No such entity exception. Error: %1", $e->getMessage()));
        } catch (LocalizedException $e) {
            throw new GraphQlInputException(__($e->getMessage()));
        }
    }

    private function getProducts($products): array
    {
        $finalProducts = [];

        foreach ($products as $product) {
            $finalProduct = $this->productRepository->getById($product['product_id']);
            $finalProduct->setQuantity($product['product_qty']);

            $finalProducts[] = $finalProduct;
        }

        return $finalProducts;
    }

    /**
     * @throws NoSuchEntityException
     * @throws LocalizedException
     */
    private function getWishlistWithItems(int $wishlistId)
    {
        $wishlist = $this->wishlistRepository->getById($wishlistId);
        $wishlist['items'] = $wishlist->getWishlistItems();

        return ['amWishlist' => $wishlist];
    }
}
