<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Multiple Wishlist GraphQL for Magento 2
 */

namespace Amasty\MultipleWishlistGraphqlByAmasty\Model\Resolver;

use Amasty\MWishlist\Model\Repository\WishlistItemRepository;
use Amasty\MWishlist\Model\Repository\WishlistRepository;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\GraphQl\Config\Element\Field;
use Magento\Framework\GraphQl\Exception\GraphQlAuthorizationException;
use Magento\Framework\GraphQl\Exception\GraphQlInputException;
use Magento\Framework\GraphQl\Query\Resolver\ArgumentsProcessorInterface;
use Magento\Framework\GraphQl\Query\Resolver\ContextInterface;
use Magento\Framework\GraphQl\Query\ResolverInterface;
use Magento\Framework\GraphQl\Schema\Type\ResolveInfo;

class DeleteProductsFromWishlist implements ResolverInterface
{
    /**
     * @var WishlistItemRepository
     */
    private $wishlistItemRepository;

    /**
     * @var WishlistRepository
     */
    private $wishlistRepository;

    /**
     * @var ArgumentsProcessorInterface
     */
    private $argsSelection;

    public function __construct(
        WishlistItemRepository $wishlistItemRepository,
        WishlistRepository $wishlistRepository,
        ArgumentsProcessorInterface $argsSelection
    ) {
        $this->wishlistItemRepository = $wishlistItemRepository;
        $this->wishlistRepository = $wishlistRepository;
        $this->argsSelection = $argsSelection;
    }

    /**
     * @param Field $field
     * @param ContextInterface $context
     * @param ResolveInfo $info
     * @param array|null $value
     * @param array|null $args
     * @return array
     * @throws GraphQlInputException
     * @throws NoSuchEntityException
     * @throws LocalizedException
     * @throws \Exception
     */
    public function resolve(
        Field $field,
        $context,
        ResolveInfo $info,
        array $value = null,
        array $args = null
    ) {
        $processedArgs = $this->argsSelection->process($info->fieldName, $args);
        $input = $processedArgs['input'];
        $customerId = $context->getUserId();

        if (null === $customerId || 0 === $customerId) {
            throw new GraphQlAuthorizationException(__('The current user cannot perform operations on wishlist'));
        }

        foreach ($input as $item) {
            try {
                $this->wishlistItemRepository->deleteItemFromWishlist(
                    $processedArgs['wishlistId'],
                    $item['wishlist_item_id'],
                    $customerId
                );
            } catch (\Exception $e) {
                throw new GraphQlInputException(__($e->getMessage()));
            }
        }

        return ['amWishlist' => $this->getWishlistWithItems($processedArgs['wishlistId'])];
    }

    /**
     * @throws NoSuchEntityException
     * @throws LocalizedException
     */
    private function getWishlistWithItems(int $wishlistId)
    {
        $wishlist = $this->wishlistRepository->getById($wishlistId);
        $wishlist['items'] = $wishlist->getWishlistItems();

        return $wishlist;
    }
}
