<?php

declare(strict_types=1);

/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Multiple Wishlist GraphQL for Magento 2
 */

namespace Amasty\MultipleWishlistGraphqlByAmasty\Model\Resolver;

use Amasty\MWishlist\Model\Repository\WishlistItemRepository;
use Amasty\MWishlist\Model\WishlistFactory;
use Amasty\MWishlist\Model\ResourceModel\Wishlist as WishlistResource;
use Amasty\MWishlist\Model\Wishlist\Item\Management as WishlistItemManagement;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\GraphQl\Config\Element\Field;
use Magento\Framework\GraphQl\Exception\GraphQlAuthorizationException;
use Magento\Framework\GraphQl\Exception\GraphQlInputException;
use Magento\Framework\GraphQl\Query\ResolverInterface;
use Magento\Framework\GraphQl\Schema\Type\ResolveInfo;
use Magento\Wishlist\Model\ItemFactory;

class MoveProductsBetweenWishlists implements ResolverInterface
{
    /**
     * @var WishlistFactory
     */
    private $wishlistFactory;

    /**
     * @var WishlistResource
     */
    private $wishlistResource;

    /**
     * @var WishlistItemManagement
     */
    private $wishlistItemManagement;

    /**
     * @var ItemFactory
     */
    private $itemFactory;

    /**
     * @var WishlistItemRepository
     */
    private $wishlistItemRepository;

    public function __construct(
        WishlistFactory $wishlistFactory,
        WishlistResource $wishlistResource,
        WishlistItemManagement $wishlistItemManagement,
        ItemFactory $itemFactory,
        WishlistItemRepository $wishlistItemRepository
    ) {
        $this->wishlistFactory = $wishlistFactory;
        $this->wishlistResource = $wishlistResource;
        $this->wishlistItemManagement = $wishlistItemManagement;
        $this->itemFactory = $itemFactory;
        $this->wishlistItemRepository = $wishlistItemRepository;
    }

    public function resolve(
        Field $field,
        $context,
        ResolveInfo $info,
        array $value = null,
        array $args = null
    ) {
        $customerId = $context->getUserId();

        /* Guest checking */
        if (null === $customerId || 0 === $customerId) {
            throw new GraphQlAuthorizationException(__('The current user cannot perform operations on wishlist'));
        }

        $sourceWishlist = $this->wishlistFactory->create();
        $this->wishlistResource->load($sourceWishlist, $args['sourceWishlistId']);

        if (null === $sourceWishlist->getId()) {
            throw new GraphQlInputException(__('The source wishlist was not found.'));
        }

        if ((int) $sourceWishlist->getCustomerId() !== $customerId) {
            throw new GraphQlAuthorizationException(
                __('The wish list is not assigned to your account and can\'t be edited.')
            );
        }

        $destinationWishlist = $this->wishlistFactory->create();
        $this->wishlistResource->load($destinationWishlist, $args['destinationWishlistId']);

        if (null === $destinationWishlist->getId()) {
            throw new GraphQlInputException(__('The destination wishlist was not found.'));
        }

        if ((int) $destinationWishlist->getCustomerId() !== $customerId) {
            throw new GraphQlAuthorizationException(
                __('The wish list is not assigned to your account and can\'t be edited.')
            );
        }

        $wishlistItems = [];
        $wishlistItemsData = $args['wishlistItems'];

        foreach ($wishlistItemsData as $wishlistItemData) {
            try {
                $item = $this->wishlistItemRepository->getWishlistItem($wishlistItemData['wishlist_item_id']);
            } catch (LocalizedException $e) {
                throw new GraphQlInputException(__($e->getMessage()));
            }

            $wishlistItems[] = $item;
        }

        foreach ($wishlistItems as $wishlistItem) {
            $this->wishlistItemManagement->move(
                $wishlistItem,
                $destinationWishlist,
                $wishlistItem->getQuantity()
            );
        }

        $destinationWishlist->setData('items', $destinationWishlist->getWishlistItems());

        return ['amWishlist' => $destinationWishlist];
    }
}
